namespace ESLintCommands {
    export let GetErrors: string = "GetErrors";
    export let FixErrors: string = "FixErrors";
}

class ESLintResponse {
    request_seq: number;
    command: string;
    version: string;
    body: string;
    error: string;
}

export class ESLintPlugin implements LanguagePlugin {
    private readonly additionalRulesDirectory?: string;
    private readonly extraOptions?: string;
    private linter?: any;
    private options?: any;
    private CliEngine?: any;
    private readonly version: string;
    private basicPath: string;

    constructor(state: PluginState) {
        this.additionalRulesDirectory = state.additionalRootDirectory;
        this.extraOptions = state.extraOptions;
        this.findESLint(state.eslintPackagePath);
        this.options = require(this.basicPath + "lib/options");
        this.CliEngine = require(this.basicPath + "lib/cli-engine");
        this.version = (this.linter == null) ? "Unknown" : this.linter.version;
    }

    onMessage(p: string, writer: MessageWriter): void {
        const request: ESLintRequest = JSON.parse(p);
        let response: ESLintResponse = new ESLintResponse();
        response.request_seq = request.seq;
        response.command = request.command;
        response.version = this.version;

        try {
            if (this.linter != null && this.options != null && this.CliEngine != null) {
                if (request.command === ESLintCommands.GetErrors) {
                    response.body = this.getErrors(request.arguments);
                } else if (request.command === ESLintCommands.FixErrors) {
                    response.body = this.fixErrors(request.arguments);
                }
            }
        } catch(e) {
            response.error = e.toString() + "\n\n" + e.stack;
        } finally {
            writer.write(JSON.stringify(response));
        }
    }

    private getErrors(getErrorsArguments: GetErrorsArguments): string {
        let args = this.createArguments(getErrorsArguments);
        const parsedOptions = this.options.parse(args);
        const cliEngine = new this.CliEngine(ESLintPlugin.translateOptions(parsedOptions));
        const report = cliEngine.executeOnText(getErrorsArguments.content, getErrorsArguments.fileName, true);
        return ESLintPlugin.formatResults(report, cliEngine, parsedOptions);
    }

    private static formatResults(report: any, cliEngine: any, parsedOptions: any | number) {
        const output = cliEngine.getFormatter(parsedOptions.format)(report.results);
        // todo: too many warnings count
        return JSON.parse(output);
    }

    private fixErrors(fixErrorsArguments: FixErrorsArguments): string {
        let args = this.createArguments(fixErrorsArguments);
        args += " --fix " + fixErrorsArguments.fileName;
        const parsedOptions = this.options.parse(args);
        const cliEngine = new this.CliEngine(ESLintPlugin.translateOptions(parsedOptions));
        const report = cliEngine.executeOnFiles(parsedOptions._);
        this.CliEngine.outputFixes(report);

        return ESLintPlugin.formatResults(report, cliEngine, parsedOptions);
    }

    private createArguments(getErrorsArguments: any) {
        let args = "";
        if (getErrorsArguments.configPath != null) {
            args += "-c \"" + getErrorsArguments.configPath + "\"";
        }
        if (this.extraOptions != null && this.extraOptions.length > 0) {
            args += " " + this.extraOptions;
        }
        if (this.additionalRulesDirectory != null && this.additionalRulesDirectory.length > 0) {
            args += " --rulesdir=\"" + this.additionalRulesDirectory + "\"";
        }
        return args;
    }

    private findESLint(eslintPackagePath: string) :any {
        if (eslintPackagePath.charAt(eslintPackagePath.length - 1) !== '/' &&
            eslintPackagePath.charAt(eslintPackagePath.length - 1) !== '\\') {
            eslintPackagePath = eslintPackagePath + '/';
        }
        eslintPackagePath = eslintPackagePath.replace("\\", "/");
        this.basicPath = eslintPackagePath;
        this.linter = require(eslintPackagePath + "lib/cli.js");
    }

    // taken from privtae part of eslint, we need it here
    /**
     * Translates the CLI options into the options expected by the CLIEngine.
     * @param {Object} cliOptions The CLI options to translate.
     * @returns {CLIEngineOptions} The options object for the CLIEngine.
     * @private
     */
    private static translateOptions(cliOptions: any): any {
        return {
            envs: cliOptions.env,
            extensions: cliOptions.ext,
            rules: cliOptions.rule,
            plugins: cliOptions.plugin,
            globals: cliOptions.global,
            ignore: cliOptions.ignore,
            ignorePath: cliOptions.ignorePath,
            ignorePattern: cliOptions.ignorePattern,
            configFile: cliOptions.config,
            rulePaths: cliOptions.rulesdir,
            useEslintrc: cliOptions.eslintrc,
            parser: cliOptions.parser,
            parserOptions: cliOptions.parserOptions,
            cache: cliOptions.cache,
            cacheFile: cliOptions.cacheFile,
            cacheLocation: cliOptions.cacheLocation,
            fix: cliOptions.fix,
            allowInlineConfig: cliOptions.inlineConfig
        };
    }
}